/*********************************************************************
** ResetEmu 1.1                                                     **
** (c) Daniel Seifert, 2001-2002                                    **
**   resetemu@dseifert.de                                           **
**   http://www.dseifert.de                                         **
**********************************************************************

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version. This 
  program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. You should have 
  received a copy of the GNU General Public License along with 
  this program; if not, write to the Free Software Foundation, 
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*********************************************************************/

#include <PalmOS.h>
#include <System/SystemMgr.h>
#include "resetemuRsc.h"

#define  APPID	 'Reem'
#define ObjPtr(frm, id)    FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, id))


DmSearchStateType stateInfoP;
Boolean           newSearch = true;
LocalID           dbID;
UInt16            cardNo;

/* here we store whether we were launched after a warm reset or not */
Boolean           warmreset = false;

/* have we already started sending the current launchcode?          */
Boolean           running = false;

/* list all the required launch codes here, every second parameter  **
** says whether we need to have done a warm reset beforehand        */
#define NUMBER_OF_CODES 4
UInt32 code_val[NUMBER_OF_CODES][2] = {
	{ sysAppLaunchCmdSystemReset,   1 },
	{ sysAppLaunchCmdSyncNotify,    0 },
	{ sysAppLaunchCmdCountryChange, 0 },
	{ sysAppLaunchCmdTimeChange,    0 }
};

/* and their names as to be shown in the list here                  */
char* code_str[NUMBER_OF_CODES] = {
	"Reset (CmdSystemReset)",
	"Hotsync (CmdSyncNotify)",
	"Country Change (CmdCountryChange)",
	"Time Change (CmdTimeChange)"
};


/* ------------------------------------------------------------------------------------------------*/


/* set the popup trigger for the SysAppLaunchCmd... list            */
static void setCmdTrigger(struct ControlType *ctrlP, Int16 selection) {
	CtlSetLabel(ctrlP, code_str[selection]);
}


/* draw the strings in the SysAppLaunchCmd... list as requested     */
static void drawCmdString(Int16 itemNum, RectangleType *bounds, char **itemText) {
	WinDrawChars(code_str[itemNum], StrLen(code_str[itemNum]), 
	             bounds->topLeft.x, bounds->topLeft.y);
}


/* setup a dynamic list                                             */
static void fillList(FormPtr frmP, UInt16 listID, UInt16 maxItems, void *drawFunction) {
	ListPtr  listP;

	listP = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, listID));
	LstSetListChoices(listP, NULL, maxItems);
	LstSetDrawFunction(listP, drawFunction);

 	LstSetHeight(listP, maxItems);
	LstSetSelection(listP, 0);
}


/* set the contents of the given field                              */
static void setfield(FormPtr frm, UInt32 ID, char* string) {
	FieldType *fldP;
	MemHandle  txtH, oldTxtH;
	void      *txtP;

	fldP    = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ID));
	oldTxtH = FldGetTextHandle(fldP);
	
	txtH = MemHandleNew(StrLen(string)+1);
	if (txtH) {
		txtP = MemHandleLock(txtH);
		StrCopy(txtP, string);
		MemHandleUnlock(txtH);

		FldSetTextHandle(fldP, txtH);
		if (oldTxtH)
			MemHandleFree(oldTxtH);
	}
}


/*------------------------------------------------------------------------------------------------*/

/* prepare everything for the next application to be notified       */
static void prepare_next() {
	FormPtr   frm;
	MemHandle nameH; 
	Err       err;
	char     *name; 

	frm = FrmGetActiveForm();
	err = DmGetNextDatabaseByTypeCreator(newSearch, 
	         &stateInfoP, sysFileTApplication, 0, true, &cardNo, &dbID);

	if (err) {
		setfield(FrmGetActiveForm(), NextLabel, "(no more available)");
		dbID = 0;
		return;
	}

	nameH = MemHandleNew(100);
	if (nameH == 0)
		return;
		
	name = (char*)MemHandleLock((MemHandle)nameH);

	DmDatabaseInfo(cardNo, dbID, name, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	setfield(frm, NextLabel, name);

	MemHandleUnlock(nameH);
	MemHandleFree(nameH);

	if (newSearch) {
		newSearch = false;
		running = false;
	} else
		running = true;
}


static Boolean Go() {
	UInt32      result;
	MemHandle   nameH;
	char*       name;
	UInt16      selection;

	/* if we have no dbID, skip it                                   */
	if (dbID == 0)
		return false;

	/* get the launchcode                                            */
	selection = LstGetSelection(ObjPtr(FrmGetActiveForm(), CmdList));
	if (code_val[selection][1] && !warmreset) {
		if (FrmCustomAlert(NeedResetAlert, 0, 0, 0) == 1)
			SysReset();
		return false;
	}

	/* retrieve the name and store it in our preferences             */	
	nameH = MemHandleNew(100);
	if (nameH == 0)
		return false;
	
	name = (char*)MemHandleLock(nameH);
	DmDatabaseInfo(cardNo, dbID, name, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	PrefSetAppPreferences(APPID, 0, 0, name, StrLen(name)+1, 1);

	/* send the launch code ... touch wood :)                        */
	SysAppLaunch(cardNo, dbID, 0, code_val[selection][0], 0, &result);

	/* update the screen, free the memory and remove the prefs       */
	setfield(FrmGetActiveForm(), LastLabel, name);
	MemHandleUnlock(nameH);
	MemHandleFree(nameH);
	PrefSetAppPreferences(APPID, 0, 0, 0, 0, 1);

	prepare_next(); 
	FrmDrawForm(FrmGetActiveForm());
	
	return true;
}


/*------------------------------------------------------------------------------------------------*/

/* Our event handler for the main form                              */
static Boolean MainFormHandleEvent (EventPtr e) {
	Boolean handled = false;
	FormPtr frm;
	UInt16  i;

	frm = FrmGetActiveForm();

	switch (e->eType) {
		case frmOpenEvent:

			setfield(frm, LastLabel, "(nothing run yet)");
			setfield(frm, NextLabel, "please wait");
			prepare_next();

			fillList(frm, CmdList, NUMBER_OF_CODES, &drawCmdString);
			LstSetSelection(ObjPtr(frm, CmdList), 0);
			setCmdTrigger(ObjPtr(frm, CmdPopTrigger), 0);

			FrmDrawForm(frm);

			handled = true;
			break;

		case menuEvent:
			switch (e->data.ctlSelect.controlID) {
				case MainFormMenuAboutID:
					FrmPopupForm(AboutForm);
					handled = true;
					break;
			}
			
			break;

		case popSelectEvent:
			if (e->data.popSelect.controlID == CmdPopTrigger) {
				setCmdTrigger(e->data.popSelect.controlP,  e->data.popSelect.selection);

				/* changing the launch code resets the app list */
				if (running) {
					FrmAlert(ResetAlert);
					newSearch = true;
					setfield(frm, LastLabel, "(nothing run yet)");
					setfield(frm, NextLabel, "please wait");
					prepare_next();
					FrmDrawForm(frm);
				}

				handled = true;
			}
			break;

		case ctlSelectEvent:
			switch (e->data.ctlSelect.controlID) {
				case NextButton:
					Go();
					handled = true;
					break;
				case NextTenButton:
					for (i=0; i<10; i++) {
						if (!Go())
							break;
					}
					handled = true;
					break;
				case SkipButton:
					prepare_next();
					FrmDrawForm(FrmGetActiveForm());
					handled = true;
					break;
			}

			break;

		default:
			break;
	}

	return handled;
}


static Boolean AboutFormHandleEvent (EventPtr e) {
	Boolean handled = false;
	FormPtr frm;

	switch (e->eType) {
		case frmOpenEvent:
			frm = FrmGetActiveForm();
			FrmDrawForm(frm);

			handled = true;
			break;

		case ctlSelectEvent:
			if (e->data.ctlSelect.controlID) {
				FrmReturnToForm(0);
				handled = true;
			}
			break;
		
		default:
			break;
	}

	return handled;
}


static Boolean ApplicationHandleEvent(EventPtr e) {
	FormPtr frm;
	UInt16  formId;
	Boolean handled = false;

	if (e->eType == frmLoadEvent) {
		formId = e->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		switch(formId) {
			case MainForm:
				FrmSetEventHandler(frm, MainFormHandleEvent);
				break;
			case AboutForm:
				FrmSetEventHandler(frm, AboutFormHandleEvent);
				break;
		}
	
		handled = true;
	}

	return handled;
}


static void EventLoop(void) {
	EventType e;
	Err       err;

	do {
		EvtGetEvent(&e, evtWaitForever);
		if (! SysHandleEvent (&e))
			if (! MenuHandleEvent (NULL, &e, &err))
				if (! ApplicationHandleEvent (&e))
					FrmDispatchEvent (&e);
	} while (e.eType != appStopEvent);
}


UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags) {
	UInt16 err = 0;
	
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		UInt32 value;
		UInt16 size;
		char  name[32];

		/* check whether we noticed a reset, if not (the feature does **
		** not yet exist) set warmreset to true                       */
		if (FtrGet(APPID, 111, &value) != 0)
			warmreset = true;
		
		size = 31;
		if (!PrefGetAppPreferences(APPID, 0, name, &size, 1)) {
			FrmCustomAlert(CrashAlert, name, 0, 0);
			PrefSetAppPreferences(APPID, 0, 0, 0, 0, 1);
		}

		FrmGotoForm(MainForm);
		EventLoop();
		FrmCloseAllForms();
	} else if (cmd == sysAppLaunchCmdSystemReset || cmd == sysAppLaunchCmdSyncNotify) {
		/* we noticed a reset or we have just been installed          */
		FtrSet(APPID, 111, 111);
	}

	return err;
}
